package com.onsiteservice.util;

import com.onsiteservice.common.annotation.sql.AutoSqlCondition;
import com.onsiteservice.util.ReflectUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.cglib.beans.BeanGenerator;


@Slf4j
public class ProxyUtil {

    /**
     * 为对象新增一个属性
     * @param object 原对象
     * @param propName 新增属性名
     * @param clazz 新增属性类型
     * @param value 新增属性值
     * @return 修改后的对象
     */
    public static Object addProperty(Object object, String propName, Class clazz, Object value) {
        BeanGenerator beanGenerator = new BeanGenerator();
        beanGenerator.setSuperclass(object.getClass());
        beanGenerator.setUseCache(true);
        beanGenerator.addProperty(propName, clazz);
        var newObj = beanGenerator.create();

        // 复制属性
        BeanUtils.copyProperties(object, newObj);
        // 新增属性值
        ReflectUtils.invokeSetter(newObj, propName, value);
        return newObj;
    }

    public static void main(String[] args) {
/*        var pojo = new Pojo(1, "Tom");
        var pojoPlus = addProperty(pojo, "age", Integer.class, 21);
        System.out.println(JSONObject.toJSONString(pojo));
        System.out.println(JSONObject.toJSONString(pojoPlus));*/

/*        var foo = new Foo(23, "Clan");
        foo.setConditions(List.of("a", "b", "c"));
        System.out.println(JSONObject.toJSONString(foo)); */

/*        var fields = BaseErrorDTO.class.getDeclaredFields();
        System.out.println(fields[0].getDeclaredAnnotation(FieldCondition.class));

        var col = CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, "pageSize");
        System.out.println(col);*/

    }


    public static class Pojo {
        private int id;
        private String name;

        public Pojo() {
        }

        public Pojo(int id, String name) {
            this.id = id;
            this.name = name;
        }

        public int getId() {
            return id;
        }

        public void setId(int id) {
            this.id = id;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }
    }

    public static class Foo implements AutoSqlCondition {
        private int id;
        private String name;
        public Foo() {
        }

        public Foo(int id, String name) {
            this.id = id;
            this.name = name;
        }

        public int getId() {
            return id;
        }

        public void setId(int id) {
            this.id = id;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }
    }


}
